<?php

namespace Drupal\taas\Token;

use Drupal\Component\Render\MarkupInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class TokenResolver
 *
 * Resolves tokens by type and name, and retrieves their values.
 */
class TokenResolver {

  /**
   * Constructs a TokenResolver object.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   * @param array $tokens
   *   An array of available tokens.
   */
  public function __construct(
    private ContainerInterface $container,
    private array $tokens = []
  ) {
  }

  /**
   * Gets the value of a token.
   *
   * @param string $type
   *   The type of the token.
   * @param string $name
   *   The name of the token.
   * @param array $data
   *   (optional) An array of data to pass to the token.
   * @param array $options
   *   (optional) An array of options to pass to the token.
   *
   * @return \Drupal\Component\Render\MarkupInterface|string|null
   *   The token value, or NULL if the token does not exist.
   */
  public function getTokenValue(
    string $type,
    string $name,
    array $data = [],
    array $options = []
  ): MarkupInterface|string|NULL {
    return $this->getToken($type, $name)
      ?->getValue($data, $options);
  }

  /**
   * Gets the available tokens.
   *
   * @return array
   *   An array of available tokens.
   */
  public function getAvailableTokens(): array {
    return $this->tokens;
  }

  /**
   * Gets a token by type and name.
   *
   * @param string $type
   *   The type of the token.
   * @param string $name
   *   The name of the token.
   *
   * @return \Drupal\taas\Token\TokenInterface|null
   *   The token object, or NULL if the token does not exist.
   *
   * @noinspection PhpIncompatibleReturnTypeInspection
   */
  private function getToken(string $type, string $name): ?TokenInterface {
    $token_service_id = $this->tokens[$type][$name]['id'] ?? NULL;

    return !is_null($token_service_id)
      && $this->container->has($token_service_id)
        ? $this->container->get($token_service_id)
        : NULL;
  }

}
